<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:e="http://software.in2p3.fr/lavoisier/entries.xsd"
                xmlns:cfg="http://software.in2p3.fr/lavoisier/application.xsd"
                xmlns:g="http://graphml.graphdrawing.org/xmlns"
                xmlns:regexp="xalan://exslt.org.regular_expressions">
    <xsl:output method="xml"/>

    <xsl:template match="/cfg:views">
        <xsl:variable name="views" select="cfg:view[not(cfg:info/cfg:category='Hidden')]"/>
        <g:graphml>
            <g:key id="rankdir" for="graph">
                <g:default>LR</g:default>
            </g:key>
            <g:key id="style" for="node">
                <g:default>filled</g:default>
            </g:key>
            <g:graph id="dependencies" edgedefault="undirected">
                <xsl:apply-templates select="$views" mode="NODE"/>
                <xsl:apply-templates select="$views" mode="EDGE"/>
            </g:graph>
        </g:graphml>
    </xsl:template>

    <xsl:template match="cfg:view" mode="NODE">
        <g:node id="{@name}">
            <xsl:for-each select="cfg:info/cfg:category">
                <g:data key="group"><xsl:value-of select="text()"/></g:data>
                <g:data key="description"><xsl:value-of select="text()"/></g:data>
            </xsl:for-each>
            <xsl:for-each select="cfg:cache">
                <g:data key="class">cached</g:data>
            </xsl:for-each>
            <g:data key="links">{"Status Details":"/lavoisier/status/jmx/domain/group/mbean[@name='<xsl:value-of select="@name"/>']"}</g:data>
            <g:data key="URL">/lavoisier/status/jmx/domain/group/mbean[@name='<xsl:value-of select="@name"/>']</g:data>
            <g:data key="details">
                <xsl:variable name="regexp">(?:[^\.]+\.)*(.*)</xsl:variable>
                <xsl:text>[</xsl:text>
                <xsl:for-each select="cfg:connector">"<xsl:value-of select="regexp:match(@type, $regexp, 'g')"/>"</xsl:for-each>
                <xsl:for-each select="cfg:serializer|cfg:validator|cfg:processors/*|cfg:cache">
                    <xsl:choose>
                        <xsl:when test="@type">,"<xsl:value-of select="regexp:match(@type, $regexp, 'g')"/>"</xsl:when>
                        <xsl:otherwise>,"XML template"</xsl:otherwise>
                    </xsl:choose>
                </xsl:for-each>
                <xsl:text>]</xsl:text>
            </g:data>
        </g:node>
    </xsl:template>

    <xsl:template match="cfg:view" mode="EDGE">
        <xsl:apply-templates select="descendant::cfg:*/@eval | descendant::cfg:*/text()" mode="EDGE"/>
    </xsl:template>
    <xsl:template match="@eval | text()" mode="EDGE">
        <xsl:variable name="target" select="ancestor::cfg:view/@name"/>
        <xsl:variable name="regexp">view(?:_path|_post|_path_post)?\s*\(\s*'(\w[\w\-\.]*)'</xsl:variable>
        <xsl:for-each select="regexp:match(., $regexp, 'g')">
            <g:edge source="{text()}" target="{$target}"/>
        </xsl:for-each>
    </xsl:template>
</xsl:stylesheet>