<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:t="http://software.in2p3.fr/lavoisier/tables.xsd">
    <xsl:output method="xml" indent="yes" xalan:indent-amount="4"/>
    <xsl:strip-space elements="*"/><!-- prevents text() function to ignore the text nodes other than 1st one -->
    <xsl:param name="view">Default rendering</xsl:param>

    <xsl:template match="/t:tables" priority="1"><t:tables title="{$view}"><xsl:apply-templates/></t:tables></xsl:template>
    <xsl:template match="t:fields" priority="1"><t:fields><xsl:apply-templates/></t:fields></xsl:template>
    <xsl:template match="t:field" priority="1"><t:field label="{@label}"><xsl:apply-templates/></t:field></xsl:template>
    <xsl:template match="t:table" priority="1"><t:table name="{@name}"><xsl:apply-templates/></t:table></xsl:template>
    <xsl:template match="t:column_labels" priority="1"><t:column_labels><xsl:apply-templates/></t:column_labels></xsl:template>
    <xsl:template match="t:column_label" priority="1"><t:column_label><xsl:apply-templates/></t:column_label></xsl:template>
    <xsl:template match="t:row" priority="1"><t:row><xsl:apply-templates/></t:row></xsl:template>
    <xsl:template match="t:column[not(@link)]" priority="1"><t:column><xsl:apply-templates/></t:column></xsl:template>
    <xsl:template match="t:column[@link]" priority="1"><t:column link="{@link}"><xsl:apply-templates/></t:column></xsl:template>

    <xsl:template match="/*">
        <t:tables title="{$view}">
            <xsl:apply-templates select="." mode="ROOT"/>
        </t:tables>
    </xsl:template>
    <xsl:template match="*[count(*[*]) = 1]" mode="ROOT">
        <xsl:apply-templates mode="ROOT"/>
    </xsl:template>
    <xsl:template match="*" mode="ROOT">
        <xsl:variable name="single" select="*[not(local-name()=local-name(preceding-sibling::*[1]) or local-name()=local-name(following-sibling::*[1]))]"/>
        <xsl:variable name="multiple"   select="*[local-name()=local-name(preceding-sibling::*[1]) or local-name()=local-name(following-sibling::*[1])]"/>
        <t:fields>
            <xsl:for-each select="@* | $single[not(*)][normalize-space(text())!='']">
                <t:field label="{local-name()}">
                    <xsl:value-of select="."/>
                </t:field>
            </xsl:for-each>
        </t:fields>
        <xsl:choose>
            <xsl:when test="$multiple">
                <xsl:apply-templates mode="TABLE"
                                     select="$multiple[not(local-name()=local-name(preceding-sibling::*[1]))]"/>
            </xsl:when>
            <xsl:otherwise>
                <xsl:apply-templates mode="TABLE"
                                     select="$single/*[not(local-name()=local-name(preceding-sibling::*[1]))]"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>

    <xsl:template match="*" mode="TABLE">
        <t:table>
            <xsl:choose>
                <xsl:when test="following-sibling::*[local-name()=local-name(current())]">
                    <!-- current element is first row -->
                    <xsl:attribute name="name"><xsl:value-of select="local-name()"/></xsl:attribute>
                    <xsl:call-template name="TABLE">
                        <xsl:with-param name="rows" select=". | following-sibling::*[local-name()=local-name(current())]"/>
                    </xsl:call-template>
                </xsl:when>
                <xsl:otherwise>
                    <!-- current element is parent of rows -->
                    <xsl:attribute name="name">
                        <xsl:choose>
                            <xsl:when test="@id|@ID"><xsl:value-of select="@id|@ID"/></xsl:when>
                            <xsl:when test="@name|@NAME"><xsl:value-of select="@name|@NAME"/></xsl:when>
                            <xsl:when test="count(@*)=1"><xsl:value-of select="@*"/></xsl:when>
                            <xsl:otherwise><xsl:value-of select="local-name()"/></xsl:otherwise>
                        </xsl:choose>
                    </xsl:attribute>
                    <xsl:call-template name="TABLE">
                        <xsl:with-param name="rows" select="*"/>
                    </xsl:call-template>
                </xsl:otherwise>
            </xsl:choose>
        </t:table>
    </xsl:template>
    <xsl:template name="TABLE">
        <xsl:param name="rows"/>
        <xsl:apply-templates select="$rows[1]" mode="HEADER"/>
        <xsl:for-each select="$rows">
            <xsl:apply-templates select="$rows[1]" mode="ROW">
                <xsl:with-param name="current" select="."/>
            </xsl:apply-templates>
        </xsl:for-each>
    </xsl:template>

    <xsl:template match="@* | *" mode="LABEL">
        <t:column_label>
            <xsl:if test="number(.)=.">
                <xsl:attribute name="unit">value</xsl:attribute>
            </xsl:if>
            <xsl:value-of select="local-name()"/>
        </t:column_label>
    </xsl:template>
    <xsl:template match="text()" mode="LABEL">
        <t:column_label>
            <xsl:if test="number(.)=.">
                <xsl:attribute name="unit">value</xsl:attribute>
            </xsl:if>
            <xsl:text>#text</xsl:text>
        </t:column_label>
    </xsl:template>
    <xsl:template match="*" mode="HEADER">
        <t:column_labels>
            <xsl:apply-templates select="@* | text()[.!='']" mode="LABEL"/>
            <xsl:for-each select="*">
                <xsl:if test="not((preceding::*|following-sibling::*)[local-name()=local-name(current())])">
                    <xsl:apply-templates select="." mode="LABEL"/>
                </xsl:if>
            </xsl:for-each>
        </t:column_labels>
    </xsl:template>
    <xsl:template match="*" mode="ROW">
        <xsl:param name="current"/>
        <t:row>
            <xsl:for-each select="@*">
                <xsl:variable name="value" select="$current/@*[local-name()=local-name(current())]"/>
                <t:column><xsl:value-of select="$value"/></t:column>
            </xsl:for-each>
            <xsl:for-each select="*">
                <xsl:if test="not((preceding::*|following-sibling::*)[local-name()=local-name(current())])">
                    <xsl:variable name="value" select="$current/*[local-name()=local-name(current())]"/>
                    <t:column><xsl:value-of select="$value"/></t:column>
                </xsl:if>
            </xsl:for-each>
            <xsl:for-each select="text()">
                <t:column><xsl:value-of select="$current/text()"/></t:column>
            </xsl:for-each>
        </t:row>
    </xsl:template>
</xsl:stylesheet>